@@ -1,20 +1,24 @@
 #include <time.h>
 
 #include <X11/Xatom.h>
+#include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xlibint.h>
 
 #include "xklavier_private.h"
+#include "xklavier_private_xkb.h"
 
 int XklFilterEvents( XEvent * xev )
 {
   XAnyEvent *pe = ( XAnyEvent * ) xev;
   XklDebug( 400, "**> Filtering event %d of type %d from window %d\n",
             pe->serial, pe->type, pe->window );
+#ifdef XKB_HEADERS_PRESENT
   if( xev->type == _xklXkbEventType )
   {
     _XklXkbEvHandler( ( XkbEvent * ) xev );
   } else
+#endif
     switch ( xev->type )
     {                           /* core events */
       case FocusIn:
@@ -117,6 +121,7 @@ void _XklStdXkbHandler( int grp, XklStateChange changeType, unsigned inds,
   _XklSaveAppState( _xklCurClient, &_xklCurState );
 }
 
+#ifdef XKB_HEADERS_PRESENT
 /**
  * XKB event handler
  */
@@ -193,6 +198,7 @@ void _XklXkbEvHandler( XkbEvent * kev )
       break;
   }
 }
+#endif
 
 /**
  * FocusIn handler
@@ -233,6 +239,16 @@ void _XklFocusInEvHandler( XFocusChangeEvent * fev )
     if( _xklCurClient != appWin )
     {
       Bool transparent;
+      XklState tmpState;
+
+      /**
+       *  For fast mouse movements - the state is probably not updated yet
+       *  (because of the group change notification being late).
+       *  so we'll enforce the update.
+       */
+      if( XklGetState( _xklCurClient, &tmpState ) )
+        _xklCurState = tmpState;
+
       _xklCurClient = appWin;
       XklDebug( 150, "CurClient:changed to " WINID_FORMAT ", '%s'\n",
                 _xklCurClient, _XklGetDebugWindowTitle( _xklCurClient ) );
@@ -273,6 +289,7 @@ void _XklFocusInEvHandler( XFocusChangeEvent * fev )
           }
         }
 
+#ifdef XKB_HEADERS_PRESENT
         if( XklGetIndicatorsHandling(  ) )
         {
           int i;
@@ -294,6 +311,7 @@ void _XklFocusInEvHandler( XFocusChangeEvent * fev )
                       selectedWindowState.indicators & bit, status );
           }
         } else
+#endif
           XklDebug( 150,
                     "Not restoring the indicators %X after gaining focus: indicator handling is not enabled\n",
                     _xklCurState.indicators );
@@ -399,6 +417,7 @@ void _XklPropertyEvHandler( XPropertyEvent * pev )
         XklDelState( pev->window );
       }
     }
+#ifdef XKB_HEADERS_PRESENT
   } else
     if( pev->atom == _xklAtoms[XKB_RF_NAMES_PROP_ATOM]
         && pev->window == _xklRootWindow )
@@ -411,6 +430,7 @@ void _XklPropertyEvHandler( XPropertyEvent * pev )
       _XklFreeAllInfo(  );
       _XklLoadAllInfo(  );
     }
+#endif
   }
 }
 
